@@ -36,16 +36,14 @@ static inline QPixmap get_icon (const char * name)
     return pm;
 }
 
-PlaylistModel::PlaylistModel (QObject * parent, int id) : QAbstractListModel (parent),
+PlaylistModel::PlaylistModel (QObject * parent, int id) :
+    QAbstractListModel (parent),
     m_uniqueId (id)
 {
     m_rows = aud_playlist_entry_count (playlist ());
 }
 
-PlaylistModel::~PlaylistModel ()
-{
-
-}
+PlaylistModel::~PlaylistModel () {}
 
 int PlaylistModel::rowCount (const QModelIndex & parent) const
 {
@@ -71,7 +69,7 @@ QVariant PlaylistModel::data (const QModelIndex &index, int role) const
         case PL_COL_ARTIST:
         case PL_COL_ALBUM:
         case PL_COL_LENGTH:
-            tuple = aud_playlist_entry_get_tuple (playlist (), index.row (), Playlist::Guess);
+            tuple = aud_playlist_entry_get_tuple (playlist (), index.row (), Playlist::NoWait);
             break;
         }
 
@@ -88,6 +86,7 @@ QVariant PlaylistModel::data (const QModelIndex &index, int role) const
         case PL_COL_LENGTH:
             return QString (str_format_time (tuple.get_int (Tuple::Length)));
         }
+        break;
 
     case Qt::TextAlignmentRole:
         switch (index.column ())
@@ -95,6 +94,7 @@ QVariant PlaylistModel::data (const QModelIndex &index, int role) const
         case PL_COL_LENGTH:
             return Qt::AlignRight;
         }
+        break;
 
     case Qt::DecorationRole:
         if (index.column () == 0 && index.row () == aud_playlist_get_position (playlist ()))
@@ -107,6 +107,7 @@ QVariant PlaylistModel::data (const QModelIndex &index, int role) const
             else
                 return get_icon ("media-playback-stop");
         }
+        break;
     }
     return QVariant ();
 }
@@ -147,6 +148,9 @@ int PlaylistModel::uniqueId () const
 
 bool PlaylistModel::insertRows (int row, int count, const QModelIndex & parent)
 {
+    if (count < 1)
+        return true;
+
     int last = row + count - 1;
     beginInsertRows (parent, row, last);
     m_rows = aud_playlist_entry_count (playlist ());
@@ -156,6 +160,9 @@ bool PlaylistModel::insertRows (int row, int count, const QModelIndex & parent)
 
 bool PlaylistModel::removeRows (int row, int count, const QModelIndex & parent)
 {
+    if (count < 1)
+        return true;
+
     int last = row + count - 1;
     beginRemoveRows (parent, row, last);
     m_rows = aud_playlist_entry_count (playlist ());
@@ -165,6 +172,9 @@ bool PlaylistModel::removeRows (int row, int count, const QModelIndex & parent)
 
 void PlaylistModel::updateRows (int row, int count)
 {
+    if (count < 1)
+        return;
+
     int bottom = row + count - 1;
     auto topLeft = createIndex (row, 0);
     auto bottomRight = createIndex (bottom, columnCount () - 1);
@@ -175,7 +185,7 @@ QString PlaylistModel::getQueued (int row) const
 {
     int at = aud_playlist_queue_find_entry (playlist (), row);
     if (at < 0)
-        return QString ("");
+        return QString ();
     else
         return QString ("#%1").arg (at + 1);
 }
