@@ -19,6 +19,7 @@
 
 #include "playlist.h"
 #include "playlist_tabs.h"
+#include "menus.h"
 
 #include <QKeyEvent>
 #include <QLineEdit>
@@ -30,6 +31,7 @@
 
 PlaylistTabs::PlaylistTabs (QWidget * parent) :
     QTabWidget (parent),
+    m_pl_menu (qtui_build_pl_menu (this)),
     m_leftbtn (nullptr),
     m_tabbar (new PlaylistTabBar (this))
 {
@@ -39,66 +41,79 @@ PlaylistTabs::PlaylistTabs (QWidget * parent) :
     m_tabbar->setFocusPolicy (Qt::NoFocus);
     setTabBar (m_tabbar);
 
-    populatePlaylists ();
+    addRemovePlaylists ();
+    updateTitles ();
     setCurrentIndex (aud_playlist_get_active ());
 
-    connect (this, &QTabWidget::currentChanged, this, &PlaylistTabs::currentChangedTrigger);
+    connect (this, & QTabWidget::currentChanged, this, & PlaylistTabs::currentChangedTrigger);
 }
 
-PlaylistTabs::~PlaylistTabs ()
+PlaylistWidget * PlaylistTabs::createWidget (int list)
 {
-    // TODO: cleanup playlists
+    int id = aud_playlist_get_unique_id (list);
+    auto widget = new PlaylistWidget (this, id);
+    widget->setContextMenu (m_pl_menu);
+    return widget;
 }
 
-void PlaylistTabs::maybeCreateTab (int count_, int uniq_id)
+void PlaylistTabs::addRemovePlaylists ()
 {
     int tabs = count ();
-
-    for (int i = 0; i < tabs; i++)
-    {
-        PlaylistWidget * playlistWidget = (PlaylistWidget *) widget (i);
-        if (uniq_id == playlistWidget->uniqueId())
-            return;
-    }
-
-    auto playlistWidget = new PlaylistWidget (0, uniq_id);
-    addTab ((QWidget *) playlistWidget, QString (aud_playlist_get_title (count_)));
-}
-
-void PlaylistTabs::cullPlaylists ()
-{
-    int tabs = count ();
-
-    for (int i = 0; i < tabs; i++)
-    {
-         PlaylistWidget * playlistWidget = (PlaylistWidget *) widget (i);
-
-         if (playlistWidget == nullptr || playlistWidget->playlist() < 0)
-         {
-             removeTab(i);
-             delete playlistWidget;
-         }
-         else
-             setTabText(i, QString (aud_playlist_get_title (playlistWidget->playlist())));
-    }
-}
-
-void PlaylistTabs::populatePlaylists ()
-{
     int playlists = aud_playlist_count ();
 
-    for (int count = 0; count < playlists; count++)
-        maybeCreateTab(count, aud_playlist_get_unique_id (count));
+    for (int i = 0; i < tabs; i ++)
+    {
+        auto widget = playlistWidget (i);
+        int playlist = widget->playlist ();
 
-    cullPlaylists();
+        if (playlist < 0)
+        {
+            removeTab (i);
+            delete widget;
+            tabs --;
+            i --;
+        }
+        else if (playlist != i)
+        {
+            bool found = false;
+
+            for (int j = i + 1; j < tabs; j ++)
+            {
+                widget = playlistWidget (j);
+                playlist = widget->playlist ();
+
+                if (playlist == i)
+                {
+                    removeTab (j);
+                    insertTab (i, widget, QString ());
+                    found = true;
+                    break;
+                }
+            }
+
+            if (! found)
+            {
+                insertTab (i, createWidget (i), QString ());
+                tabs ++;
+            }
+        }
+    }
+
+    while (tabs < playlists)
+    {
+        addTab (createWidget (tabs), QString ());
+        tabs ++;
+    }
 }
 
-PlaylistWidget * PlaylistTabs::playlistWidget (int num)
+void PlaylistTabs::updateTitles ()
 {
-    return (PlaylistWidget *) widget (num);
+    int tabs = count ();
+    for (int i = 0; i < tabs; i ++)
+        setTabTitle (i, aud_playlist_get_title (i));
 }
 
-void PlaylistTabs::filterTrigger (const QString &text)
+void PlaylistTabs::filterTrigger (const QString & text)
 {
     ((PlaylistWidget *) currentWidget ())->setFilter (text);
 }
@@ -114,11 +129,22 @@ QLineEdit * PlaylistTabs::getTabEdit (int idx)
     return dynamic_cast<QLineEdit *> (m_tabbar->tabButton (idx, QTabBar::LeftSide));
 }
 
-void PlaylistTabs::setupTab (int idx, QWidget * button, const QString & text, QWidget * * oldp)
+void PlaylistTabs::setTabTitle (int idx, const char * text)
+{
+    // escape ampersands for setTabText ()
+    auto title = QString (text).replace ("&", "&&");
+
+    if (aud_get_bool ("qtui", "entry_count_visible"))
+        title += QString (" (%1)").arg (aud_playlist_entry_count (idx));
+
+    setTabText (idx, title);
+}
+
+void PlaylistTabs::setupTab (int idx, QWidget * button, const char * text, QWidget * * oldp)
 {
     QWidget * old = m_tabbar->tabButton (idx, QTabBar::LeftSide);
     m_tabbar->setTabButton (idx, QTabBar::LeftSide, button);
-    setTabText (idx, text);
+    setTabTitle (idx, text);
 
     if (oldp)
         * oldp = old;
@@ -139,8 +165,8 @@ void PlaylistTabs::tabEditedTrigger ()
     if (! edit)
         return;
 
-    QString title = edit->text ();
-    aud_playlist_set_title (idx, title.toUtf8 ());
+    QByteArray title = edit->text ().toUtf8 ();
+    aud_playlist_set_title (idx, title);
 
     setupTab (idx, m_leftbtn, title, nullptr);
     m_leftbtn = nullptr;
@@ -148,11 +174,17 @@ void PlaylistTabs::tabEditedTrigger ()
 
 void PlaylistTabs::editTab (int idx)
 {
-    QLineEdit * edit = new QLineEdit (tabText (idx));
+    QLineEdit * edit = getTabEdit (idx);
 
-    connect (edit, & QLineEdit::returnPressed, this, & PlaylistTabs::tabEditedTrigger);
+    if (! edit)
+    {
+        edit = new QLineEdit ((const char *) aud_playlist_get_title (idx));
 
-    setupTab (idx, edit, QString (), & m_leftbtn);
+        connect (edit, & QLineEdit::returnPressed, this, & PlaylistTabs::tabEditedTrigger);
+
+        setupTab (idx, edit, nullptr, & m_leftbtn);
+        setTabText (idx, nullptr);
+    }
 
     edit->selectAll ();
     edit->setFocus ();
@@ -162,9 +194,9 @@ bool PlaylistTabs::eventFilter (QObject * obj, QEvent * e)
 {
     if (e->type() == QEvent::KeyPress)
     {
-        QKeyEvent *ke = (QKeyEvent *) e;
+        QKeyEvent * ke = (QKeyEvent *) e;
 
-        if (ke->key() == Qt::Key_Escape)
+        if (ke->key () == Qt::Key_Escape)
         {
             cancelRename ();
             return true;
@@ -174,6 +206,16 @@ bool PlaylistTabs::eventFilter (QObject * obj, QEvent * e)
     return QTabWidget::eventFilter(obj, e);
 }
 
+void PlaylistTabs::renameCurrent ()
+{
+    int idx = currentIndex ();
+
+    if (! m_tabbar->isVisible ())
+        audqt::playlist_show_rename (idx);
+    else
+        editTab (idx);
+}
+
 void PlaylistTabs::cancelRename ()
 {
     for (int i = 0; i < count (); i ++)
@@ -187,10 +229,18 @@ void PlaylistTabs::cancelRename ()
     }
 }
 
+void PlaylistTabs::playlist_activate_cb ()
+{
+    if (! aud_playlist_update_pending ())
+        setCurrentIndex (aud_playlist_get_active ());
+}
+
 void PlaylistTabs::playlist_update_cb (Playlist::UpdateLevel global_level)
 {
     if (global_level == Playlist::Structure)
-        populatePlaylists ();
+        addRemovePlaylists ();
+    if (global_level >= Playlist::Metadata)
+        updateTitles ();
 
     int lists = aud_playlist_count ();
     for (int list = 0; list < lists; list ++)
@@ -199,6 +249,8 @@ void PlaylistTabs::playlist_update_cb (Playlist::UpdateLevel global_level)
         if (update.level)
             playlistWidget (list)->update (update);
     }
+
+    setCurrentIndex (aud_playlist_get_active ());
 }
 
 void PlaylistTabs::playlist_position_cb (int list)
@@ -210,30 +262,58 @@ void PlaylistTabs::playlist_position_cb (int list)
 
 PlaylistTabBar::PlaylistTabBar (QWidget * parent) : QTabBar (parent)
 {
+    setMovable (true);
     setDocumentMode (true);
-    setTabsClosable (true);
+    updateSettings ();
 
-    connect (this, &QTabBar::tabCloseRequested, this, &PlaylistTabBar::handleCloseRequest);
+    connect (this, & QTabBar::tabMoved, this, & PlaylistTabBar::tabMoved);
+    connect (this, & QTabBar::tabCloseRequested, this, & PlaylistTabBar::handleCloseRequest);
 }
 
-void PlaylistTabBar::mouseDoubleClickEvent (QMouseEvent *e)
+void PlaylistTabBar::tabMoved (int from, int to)
 {
-    PlaylistTabs *p = (PlaylistTabs *) parent();
+    aud_playlist_reorder (from, to, 1);
+}
 
-    int idx = tabAt (e->pos());
-    if (idx < 0)
+void PlaylistTabBar::mousePressEvent (QMouseEvent * e)
+{
+    if (e->button () == Qt::MidButton)
+    {
+        int index = tabAt (e->pos ());
+        handleCloseRequest (index);
+        e->accept ();
+    }
+
+    QTabBar::mousePressEvent (e);
+}
+
+void PlaylistTabBar::mouseDoubleClickEvent (QMouseEvent * e)
+{
+    int idx = tabAt (e->pos ());
+    if (idx < 0 || e->button () != Qt::LeftButton)
         return;
 
-    p->editTab (idx);
+    PlaylistTabs * p = (PlaylistTabs *) parent ();
+    PlaylistWidget * pl = p->playlistWidget (idx);
+
+    aud_playlist_play (pl->playlist ());
 }
 
 void PlaylistTabBar::handleCloseRequest (int idx)
 {
-    PlaylistTabs *p = (PlaylistTabs *) parent ();
-    PlaylistWidget *pl = (PlaylistWidget *) p->widget (idx);
+    PlaylistTabs * p = (PlaylistTabs *) parent ();
+    PlaylistWidget * pl = p->playlistWidget (idx);
 
     if (! pl)
         return;
 
     audqt::playlist_confirm_delete (pl->playlist ());
 }
+
+void PlaylistTabBar::updateSettings ()
+{
+#if QT_VERSION >= 0x050400
+    setAutoHide (! aud_get_bool ("qtui", "playlist_tabs_visible"));
+#endif
+    setTabsClosable (aud_get_bool ("qtui", "close_button_visible"));
+}
